#include <QtGui>
#include "rowedlist.h"

RowedList::RowedList(QWidget *parent)
        : QWidget(parent)
{
    setSizePolicy(QSizePolicy::Ignored, QSizePolicy::MinimumExpanding);

    curFont=parent->font();
    curHeaderFont=parent->font();

    //    
    QFontMetrics fm(curFont);
    curRowHeight=fm.height()+3;
    curCellWidth=curRowHeight; // 
    curHeaderWidth=5; // 5   
    numberOfCols=5;
    numberOfRows=2;
    curCount=0;
    lastKey = qMakePair(-1,-1);
}

QSize RowedList::sizeHint() const
{
    return QSize(curHeaderWidth+3+numberOfCols*curCellWidth,curRowHeight*numberOfRows+3);
}

void RowedList::clear()
{
    QStringList l;
    setItems(l);
    lastKey = qMakePair(-1,-1);
}

void RowedList::resizeEvent(QResizeEvent *event)
{
    numberOfCols=qMax((this->width()-curHeaderWidth) / curCellWidth,2); //  5 
    //  
    headersPos.clear();
    headersPos.append(0);
    if (curCount>0) {
        int j=0;
        for (int i=0;i<curCount;i++) {
            QString s=items.at(i);
            int x=s.length()/numberOfCols;
            if (s.length()-x*numberOfCols!=0)
                j+=x+1;
            else j+=x;
            headersPos.append(j);
        }
        numberOfRows=j;
    }
    else numberOfRows=1;

    update();
}

void RowedList::setRowHeight(int newRowHeight)
{
    curRowHeight=newRowHeight;
    adjustSize();
    update();
    updateGeometry();
}

void RowedList::setCellWidth(int newCellWidth)
{
    curCellWidth=newCellWidth;
    adjustSize();
    update();
    updateGeometry();
}

void RowedList::setHeaderWidth(int newHeaderWidth)
{
    curHeaderWidth=newHeaderWidth;
    adjustSize();
    update();
    updateGeometry();
}

//  - ,   
void RowedList::setFont(const QFont &newFont)
{
    curFont=newFont;
    QFontMetrics fm(curFont);
    curRowHeight=fm.height()+3;
    curCellWidth=curRowHeight;
    update();
    updateGeometry();
}

void RowedList::setItems(const QStringList &list)
{
    items.clear();
    headers.clear();
    curCount=list.count();

    if (curCount>0) {
        QFontMetrics fm(curHeaderFont);
        //   
        int w=0;
        int curw=0;
        for (int i=0; i<curCount;i++) {
            QStringList l=list.at(i).split("|");
            items.append(l.at(1));
            headers.append(l.at(0));
            curw=fm.width(l.at(0));
            if (curw>w) w=curw;
        }
        if (w>0)
            curHeaderWidth=w+5;
        else curHeaderWidth=0;
    }

    adjustSize();
    update();
    updateGeometry();
}

void RowedList::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    QPen pen;
    QPen pen1(Qt::gray, 1, Qt::DotLine, Qt::SquareCap, Qt::BevelJoin);
    QVector<qreal> dashes;
    dashes << 3 << 1;
    pen1.setDashPattern(dashes);
    painter.setPen(palette().foreground().color());
    painter.setBrush(Qt::white);
    painter.drawRect(0,0,width(),height());
    if (curCount>0) {
        painter.drawRect(0,0,curHeaderWidth+numberOfCols*curCellWidth,numberOfRows*curRowHeight);
        painter.drawLine(curHeaderWidth,0,curHeaderWidth,numberOfRows*curRowHeight);
        // 
        painter.setPen(pen1);
        for (int i=1;i<numberOfRows;i++) {
            painter.drawLine(curHeaderWidth+1,i*curRowHeight,curHeaderWidth+numberOfCols*curCellWidth-1,i*curRowHeight);
        }
        // 
        for (int i=1;i<numberOfCols;i++) {
            painter.drawLine(curHeaderWidth+i*curCellWidth,1,curHeaderWidth+i*curCellWidth,numberOfRows*curRowHeight-1);
        }
        // 
        painter.setFont(curHeaderFont);
        painter.setPen(pen);
        for (int i=0;i<curCount;i++) {
            painter.drawText(QRect(0,headersPos.at(i)*curRowHeight,curHeaderWidth,curRowHeight),
                             Qt::AlignCenter, headers.at(i));
            if (i!=0)
                painter.drawLine(0,headersPos.at(i)*curRowHeight,curHeaderWidth+numberOfCols*curCellWidth
                                 ,headersPos.at(i)*curRowHeight);
        }
        // 
        painter.setFont(curFont);
        for (int i=0; i<curCount; i++) {
            QString s=items.at(i);
            int lCount=s.length();
            for (int j=0; j<lCount; j++) {
                int p=j / numberOfCols;
                if (lastKey.first==i && lastKey.second==j)
                    painter.fillRect(curHeaderWidth+(j-p*numberOfCols)*curCellWidth+1,
                                     (headersPos.at(i)+p)*curRowHeight+1,
                                     curCellWidth-1,curRowHeight-1, QBrush(Qt::lightGray));
                painter.drawText(QRect(curHeaderWidth+(j-p*numberOfCols)*curCellWidth,//  
                                       (headersPos.at(i)+p)*curRowHeight,//   
                                       curCellWidth,curRowHeight),
                                 Qt::AlignCenter, s.at(j));
            }
        }
    }
}

void RowedList::mousePressEvent(QMouseEvent *event)
{
    int y=event->y();
    int x=event->x();
    if (event->button() == Qt::LeftButton) {
        if (x<=curHeaderWidth+1 || x>=curHeaderWidth+numberOfCols*curCellWidth ||
                y>=numberOfRows*curRowHeight || curCount==0) return;

        int j= (x-curHeaderWidth)/curCellWidth;
        int i=y/curRowHeight;
        int i2=0;
        for (int i1=0; i1<headersPos.count();i1++) {
            i2=i1;
            if (i<headersPos.at(i1)) break;
        }
        i2--;
        int j2=(i-headersPos.at(i2))*numberOfCols+j;
        QString s=items.at(i2);
        if (j2<s.length()) {
            lastKey=qMakePair(i2,j2);
            emit characterSelected(QString(s.at(j2)));
        }
        update();
    }
    else
        QWidget::mousePressEvent(event);
}

